@@ -1,62 +1,103 @@
 #' htmlwidget for d3.js scatter plot matrix
 #'
-#' @param data 
+#' @param data
 #'   data.frame with data to use in the chart.
-#' @param controlWidgets 
-#'   Tells if some widgets must be available to control plot; 
-#'   \code{NULL} is allowed, meaning that '!HTMLWidgets.shinyMode' is to use; 
-#'   default value is \code{FALSE}.
-#' @param categorical 
-#'   List of list (one for each data column) containing the name of available categories, 
-#'   or \code{NULL} if column corresponds to continuous data; 
+#' @param categorical
+#'   List of list (one for each data column) containing the name of available categories,
+#'   or \code{NULL} if column corresponds to continuous data;
 #'   \code{NULL} is allowed, meaning all columns are continuous.
-#' @param inputColumns 
-#'   List of boolean (one for each data column), \code{TRUE} for an input column, \code{FALSE} for an output column; 
+#' @param inputColumns
+#'   List of boolean (one for each data column), \code{TRUE} for an input column, \code{FALSE} for an output column;
 #'   \code{NULL} is allowed, meaning all columns are inputs.
-#' @param keptColumns 
-#'   List of boolean (one for each data column), \code{FALSE} if column has to be ignored; 
+#' @param keptColumns
+#'   List of boolean (one for each data column), \code{FALSE} if column has to be ignored;
 #'   \code{NULL} is allowed, meaning all columns are available.
-#' @param zAxisDim 
-#'   Name of the column represented by z axis (used to determine the color to attribute to a point); 
+#' @param cutoffs
+#'   List of 'SpCutoff'; a 'SpCutoff' is a list defining a 'xDim', 'yDim' and a list of 'xyCutoff';
+#'   a 'xyCutoff' is a pair of 'cutoff' (one for x axis, one for y axis);
+#'   a 'cutoff' is a list containing two values (min and max values)
+#'   or \code{NULL} if there is no cutoff to apply for this axis;
+#'   \code{NULL} is allowed, meaning there is no cutoff to apply.
+#' @param zAxisDim
+#'   Name of the column represented by z axis (used to determine the color to attribute to a point);
 #'   \code{NULL} is allowed, meaning there is no coloring to apply.
-#' @param distribType 
+#' @param distribType
 #'   Binary code indicating the type of distribution plot (bit 1: density plot, bit 2: histogram).
-#' @param regressionType 
+#' @param regressionType
 #'   Binary code indicating the type of regression plot (bit 1: linear, bit 2: loess).
-#' @param corrPlotType 
-#'   String indicating the type of correlation plots to use;
-#'   supported values: \code{Circles} to use a circle tree map, \code{Text} to use simple text labels; 
+#' @param corrPlotType
+#'   String indicating the type of correlation plots to use.
+#'   Supported values: \code{Circles} to use a circle tree map;
+#'   \code{Text} to display values of correlation as colored text labels (color scale domain is [-1; 1]);
+#'   \code{AbsText} to display values of correlation as colored text labels (color scale domain is [0; 1],
+#'   absolute value of correlations is used);
+#'   \code{Empty} to not display values of correlation;
 #'   default value is \code{Circles}.
-#' @param corrPlotCS 
-#'   Name of the color Scale to use for correlation plot when plot type is 'Text';
-#'   supported names: "Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault", 
-#'   "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd", "PuBuGn","PuBu", 
-#'   "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd"; 
-#'   default value is \code{Plasma}.
-#' @param rotateTitle 
+#' @param corrPlotCS
+#'   Name of the color Scale to use for correlation plot when plot type is 'Text' or 'AbsText';
+#'   supported names: "Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault",
+#'   "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd", "PuBuGn","PuBu",
+#'   "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd";
+#'   default value is \code{NULL},
+#'   which corresponds to "RdBu" if corrPlotType is \code{Text},
+#'   or "Blues" if corrPlotType is \code{AbsText}.
+#' @param rotateTitle
 #'   \code{TRUE} if column title must be rotated.
-#' @param columnLabels 
-#'   List of string (one for each data column) to display in place of column name found in data, 
-#'   or \code{NULL} if there is no alternative name; 
-#'   \code{NULL} is allowed, meaning all columns are without alternative name; 
+#' @param columnLabels
+#'   List of string (one for each data column) to display in place of column name found in data,
+#'   or \code{NULL} if there is no alternative name;
+#'   \code{NULL} is allowed, meaning all columns are without alternative name;
 #'   \code{<br>} can be used to insert line breaks.
-#' @param continuousCS 
+#' @param continuousCS
 #'   Name of the color Scale to use for continuous data;
-#'   supported names: "Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault", 
-#'   "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd", "PuBuGn","PuBu", 
-#'   "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd"; 
+#'   supported names: "Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault",
+#'   "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd", "PuBuGn","PuBu",
+#'   "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd";
 #'   default value is \code{Viridis}.
-#' @param categoricalCS 
-#'   Name of the color Scale to use for categorical data; 
-#'   supported names: Category10, Accent, Dark2, Paired, Set1; 
+#' @param categoricalCS
+#'   Name of the color Scale to use for categorical data;
+#'   supported names: Category10, Accent, Dark2, Paired, Set1;
 #'   default value is \code{Category10}.
-#' @param eventInputId 
+#' @param eventInputId
 #'   When plot event occured, reactive input to write to; \code{NULL} is allowed, default value is 'plotEvent'.
-#' @param width 
+#'   An event is a list with two named elements 'type' and 'value'.
+#'   If 'type' is 'zAxisChange', it means the coloration of points has changed,
+#'   (probably because an header of column has been clicked), and 'value' is a name of column (or NULL).
+#' @param controlWidgets
+#'   Tells if some widgets must be available to control plot;
+#'   \code{NULL} is allowed, meaning that '!HTMLWidgets.shinyMode' is to use;
+#'   default value is \code{FALSE}.
+#' @param cssRules
+#'   CSS rules to add.
+#'   Must be a named list of the form list(selector = declarations),
+#'   where selector is a valid CSS selector and declarations is a string or vector of declarations.
+#' @param plotProperties
+#'   Adjust some properties which can not be set through CSS (mainly size, color and opacity of points).
+#'   Default value is \code{NULL} which is equivalent to:
+#'     list(
+#'       noCatColor = "#43665e",
+#'       watermarkColor = "#ddd",
+#'       point = list(
+#'         alpha = 0.5,
+#'         radius = 2
+#'       ),
+#'       regression = list(
+#'         strokeWidth = 4
+#'       )
+#'     )
+#' @param slidersPosition
+#'   Set initial position of sliders, specifying which columns intervals are visible.
+#'   Default value is \code{NULL} which is equivalent to:
+#'     list(
+#'       dimCount = 8,
+#'       xStartingDimIndex = 1,
+#'       yStartingDimIndex = 1
+#'     )
+#' @param width
 #'   Integer in pixels defining the width of the widget.
-#' @param height 
+#' @param height
 #'   Integer in pixels defining the height of the widget.
-#' @param elementId 
+#' @param elementId
 #'   Unique \code{CSS} selector id for the widget.
 #'
 #' @examples
@@ -80,28 +121,49 @@
 #'    columnLabels <- gsub("\\.", "<br>", colnames(iris))
 #'    scatterPlotMatrix(iris, zAxisDim = "Species", columnLabels = columnLabels)
 #'    # Given names are displayed in place of dataset column names; <br> is used to insert line breaks
+#'
+#'    scatterPlotMatrix(iris, cssRules = list(
+#'        ".jitterZone" = "fill: pink",
+#'        ".tick text" = c("fill: red", "font-size: 1.8em")
+#'    ))
+#'    # Background of plot is pink and text of axes ticks is red and greater
+#'
+#'    scatterPlotMatrix(iris, plotProperties = list(
+#'      noCatColor = "DarkCyan",
+#'      point = list(
+#'        alpha = 0.3,
+#'        radius = 4
+#'      )
+#'    ))
+#'    # Points of plots are different:
+#'    # two times greater, with opacity reduced from 0.5 to 0.3, and a 'DarkCyan' color
+#'
 #'  }
 #'
 #' @importFrom htmlwidgets createWidget sizingPolicy shinyWidgetOutput shinyRenderWidget
 #'
 #' @export
 scatterPlotMatrix <- function(
-  data, 
-  controlWidgets = FALSE, 
-  categorical = NULL, 
-  inputColumns = NULL, 
-  keptColumns = NULL, 
-  zAxisDim = NULL, 
-  distribType = 2, 
-  regressionType = 0, 
-  corrPlotType = 'Circles', 
-  corrPlotCS = 'Plasma', 
-  rotateTitle = FALSE, 
-  columnLabels = NULL, 
-  continuousCS = 'Viridis', 
-  categoricalCS = 'Category10', 
-  eventInputId = NULL, 
-  width = NULL, 
+  data,
+  categorical = NULL,
+  inputColumns = NULL,
+  cutoffs = NULL,
+  keptColumns = NULL,
+  zAxisDim = NULL,
+  distribType = 2,
+  regressionType = 0,
+  corrPlotType = "Circles",
+  corrPlotCS = NULL,
+  rotateTitle = FALSE,
+  columnLabels = NULL,
+  continuousCS = "Viridis",
+  categoricalCS = "Category10",
+  eventInputId = NULL,
+  controlWidgets = FALSE,
+  cssRules = NULL,
+  plotProperties = NULL,
+  slidersPosition = NULL,
+  width = NULL,
   height = NULL,
   elementId = NULL
 ) {
@@ -110,9 +172,9 @@ scatterPlotMatrix <- function(
     list(
       data = data,
       rowLabels = rownames(data),
-      controlWidgets = controlWidgets,
       categorical = categorical,
       inputColumns = inputColumns,
+      cutoffs = cutoffs,
       keptColumns = keptColumns,
       zAxisDim = zAxisDim,
       distribType = distribType,
@@ -123,18 +185,22 @@ scatterPlotMatrix <- function(
       columnLabels = columnLabels,
       continuousCS = continuousCS,
       categoricalCS = categoricalCS,
-      eventInputId = eventInputId
+      eventInputId = eventInputId,
+      controlWidgets = controlWidgets,
+      cssRules = cssRules,
+      plotProperties = plotProperties,
+      slidersPosition = slidersPosition
     )
   )
 
   # create widget
   htmlwidgets::createWidget(
-    name = 'scatterPlotMatrix',
+    name = "scatterPlotMatrix",
     args,
     width = width,
     height = height,
-    sizingPolicy = htmlwidgets::sizingPolicy(padding = 0, browser.defaultHeight = 960, browser.defaultWidth = 'auto'),
-    package = 'scatterPlotMatrix',
+    sizingPolicy = htmlwidgets::sizingPolicy(padding = 0, browser.defaultHeight = 960, browser.defaultWidth = "auto"),
+    package = "scatterPlotMatrix",
     elementId = elementId
   )
 }
@@ -156,34 +222,42 @@ scatterPlotMatrix <- function(
 #' @name scatterPlotMatrix-shiny
 #'
 #' @export
-scatterPlotMatrixOutput <- function(outputId, width = '100%', height = '600px'){
-  htmlwidgets::shinyWidgetOutput(outputId, 'scatterPlotMatrix', width, height, package = 'scatterPlotMatrix')
+scatterPlotMatrixOutput <- function(outputId, width = "100%", height = "600px") {
+  htmlwidgets::shinyWidgetOutput(outputId, "scatterPlotMatrix", width, height, package = "scatterPlotMatrix")
 }
 
 #' @rdname scatterPlotMatrix-shiny
 #' @export
 renderScatterPlotMatrix <- function(expr, env = parent.frame(), quoted = FALSE) {
-  if (!quoted) { expr <- substitute(expr) } # force quoted
+  if (!quoted) {
+    expr <- substitute(expr)
+  } # force quoted
   htmlwidgets::shinyRenderWidget(expr, scatterPlotMatrixOutput, env, quoted = TRUE)
 }
 
 checkSpmArgs <- function(args) {
-  return (
+  return(
     checkEventInputId(
-      checkCategoricalCS(
-        checkContinuousCS(
-          checkColumnLabels(
-            checkColumnLabels(
-              checkRotateTitle(
-                checkRegressionType(
-                  checkCorrPlotCS(
-                    checkCorrPlotType(
-                      checkDistribType(
-                        checkZAxisDim(
-                          checkInputColumns(
-                            checkKeptColumns(
-                              checkCategorical(
-                                checkData(args)
+      checkSlidersPosition(
+        checkScatterPlotProperties(
+          checkCategoricalCS(
+            checkContinuousCS(
+              checkColumnLabels(
+                checkColumnLabels(
+                  checkRotateTitle(
+                    checkRegressionType(
+                      checkCorrPlotCS(
+                        checkCorrPlotType(
+                          checkDistribType(
+                            checkZAxisDim(
+                              checkInputColumns(
+                                checkCutoffs(
+                                  checkKeptColumns(
+                                    checkCategorical(
+                                      checkData(args)
+                                    )
+                                  )
+                                )
                               )
                             )
                           )
@@ -296,10 +370,34 @@ checkInputColumns <- function(args) {
   return(args)
 }
 
+checkCutoffs <- function(args) {
+  if (!is.null(args$cutoffs) && !is.list(args$cutoffs)) {
+    message("'cutoffs' must be a list")
+    args["cutoffs"] <- list(NULL)
+  }
+  if (!is.null(args$cutoffs)) {
+    expectedNames <- c("xDim", "yDim", "xyCutoffs")
+    for (i in seq_len(length(args$cutoffs))) {
+      ## if cutoffs are provided for current column
+      if (!setequal(intersect(expectedNames, names(args$cutoffs[[i]])), expectedNames)) {
+        message(paste("cutoff", i, "doesn't contains expected fields:", toString(expectedNames)))
+        args["cutoffs"] <- list(NULL)
+        break
+      }
+    }
+  }
+  return(args)
+}
+
 checkZAxisDim <- function(args) {
   colNames <- colnames(args$data)
   if (!is.null(args$zAxisDim) && is.na(match(args$zAxisDim, colNames))) {
-    message(paste("zAxisDim:", args$zAxisDim, "must be a valid column dimension, it must be one of:", toString(colNames)))
+    message(paste(
+      "zAxisDim:",
+      args$zAxisDim,
+      "must be a valid column dimension, it must be one of:",
+      toString(colNames)
+    ))
     args["zAxisDim"] <- list(NULL)
   }
   return(args)
@@ -334,9 +432,14 @@ checkRegressionType <- function(args) {
 }
 
 checkCorrPlotType <- function(args) {
-  corrPlotTypes <- c("Circles", "Text")
+  corrPlotTypes <- c("Empty", "Circles", "Text", "AbsText")
   if (is.na(match(args$corrPlotType, corrPlotTypes))) {
-    message(paste("corrPlotType:", args$corrPlotType, "must be a valid correlation plot type, it must be one of:", toString(corrPlotTypes)))
+    message(paste(
+      "corrPlotType:",
+      args$corrPlotType,
+      "must be a valid correlation plot type, it must be one of:",
+      toString(corrPlotTypes)
+    ))
     args$corrPlotType <- corrPlotTypes[1]
   }
   return(args)
@@ -344,13 +447,18 @@ checkCorrPlotType <- function(args) {
 
 checkCorrPlotCS <- function(args) {
   continuousCSList <- c(
-    "Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault", 
-    "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd", 
-    "PuBuGn","PuBu", "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd"
+    "Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault",
+    "Blues", "Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd",
+    "PuBuGn", "PuBu", "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd"
   )
-  if (is.na(match(args$corrPlotCS, continuousCSList))) {
-    message(paste("corrPlotCS:", args$corrPlotCS, "must be a valid continuous color scale name, it must be one of:", toString(continuousCSList)))
-    args$corrPlotCS <- continuousCSList[4]
+  if (!is.null(args$corrPlotCS) && is.na(match(args$corrPlotCS, continuousCSList))) {
+    message(paste(
+      "corrPlotCS:",
+      args$corrPlotCS,
+      "must be a valid continuous color scale name, it must be one of:",
+      toString(continuousCSList)
+    ))
+    args$corrPlotCS <- NULL
   }
   return(args)
 }
@@ -388,12 +496,17 @@ checkColumnLabels <- function(args) {
   
 checkContinuousCS <- function(args) {
   continuousCSList <- c(
-    "Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault", 
-    "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd", 
-    "PuBuGn","PuBu", "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd"
+    "Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault",
+    "Blues", "Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd",
+    "PuBuGn", "PuBu", "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd"
   )
   if (is.na(match(args$continuousCS, continuousCSList))) {
-    message(paste("continuousCS:", args$continuousCS, "must be a valid continuous color scale name, it must be one of:", toString(continuousCSList)))
+    message(paste(
+      "continuousCS:",
+      args$continuousCS,
+      "must be a valid continuous color scale name, it must be one of:",
+      toString(continuousCSList)
+    ))
     args$continuousCS <- continuousCSList[1]
   }
   return(args)
@@ -402,12 +515,51 @@ checkContinuousCS <- function(args) {
 checkCategoricalCS <- function(args) {
   categoricalCSList <- c("Category10", "Accent", "Dark2", "Paired", "Set1")
   if (is.na(match(args$categoricalCS, categoricalCSList))) {
-    message(paste("categoricalCS:", args$categoricalCS, "must be a valid categorical color scale name, it must be one of:", toString(categoricalCSList)))
+    message(paste(
+      "categoricalCS:",
+      args$categoricalCS,
+      "must be a valid categorical color scale name, it must be one of:",
+      toString(categoricalCSList)
+    ))
     args["categoricalCS"] <- categoricalCSList[1]
   }
   return(args)
 }
   
+checkScatterPlotProperties <- function(args) {
+  if (!is.null(args$plotProperties) && !is.list(args$plotProperties)) {
+    message("'plotProperties' must be a list")
+    args["plotProperties"] <- list(NULL)
+  }
+  if (!is.null(args$plotProperties)) {
+    validKeys <- c("noCatColor", "watermarkColor", "point", "regression")
+    unknownKeys <- setdiff(names(args$plotProperties), validKeys)
+    if (length(unknownKeys) != 0) {
+      message(paste0("plotProperties constains invalid properties: ", toString(unknownKeys),
+        ". Valid properties are:", toString(validKeys)))
+      args["plotProperties"] <- list(NULL)
+    }
+  }
+  return(args)
+}
+
+checkSlidersPosition <- function(args) {
+  if (!is.null(args$slidersPosition) && !is.list(args$slidersPosition)) {
+    message("'slidersPosition' must be a list")
+    args["slidersPosition"] <- list(NULL)
+  }
+  if (!is.null(args$slidersPosition)) {
+    validKeys <- c("dimCount", "xStartingDimIndex", "yStartingDimIndex")
+    unknownKeys <- setdiff(names(args$slidersPosition), validKeys)
+    if (length(unknownKeys) != 0) {
+      message(paste0("slidersPosition constains invalid properties: ", toString(unknownKeys),
+        ". Valid properties are:", toString(validKeys)))
+      args["slidersPosition"] <- list(NULL)
+    }
+  }
+  return(args)
+}
+
 checkEventInputId <- function(args) {
   if (!is.null(args$eventInputId) && !is.character(args$eventInputId)) {
     message("'eventInputId' must be of character type")
@@ -417,12 +569,12 @@ checkEventInputId <- function(args) {
 }
 
 #' Tells which type of representation to use for distribution plots.
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param distribType 
+#' @param distribType
 #'   Binary code indicating the type of distribution plot (bit 1: histogram, bit 2: density plot).
 #'
-#' @return 
+#' @return
 #'   No return value, called from shiny applications for side effects.
 #'
 #' @examples
@@ -432,9 +584,9 @@ checkEventInputId <- function(args) {
 #'
 #'    ui <- fluidPage(
 #'      selectInput(
-#'        "distribType", 
-#'        "Distribution Representation:", 
-#'        choices = list("Histogram" = 2, "Density Plot" = 1), 
+#'        "distribType",
+#'        "Distribution Representation:",
+#'        choices = list("Histogram" = 2, "Density Plot" = 1),
 #'        selected = 2
 #'      ),
 #'      p("The selector controls type of representation to use for distribution plots"),
@@ -455,17 +607,17 @@ checkEventInputId <- function(args) {
 #'
 #' @export
 setDistribType <- function(id, distribType) {
-  method <- "setDistribType"
+  method <- "setDistribType" # nolint
   callJS()
 }
 
 #' Tells which type of regression to use for regression plots.
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param regressionType 
+#' @param regressionType
 #'   Binary code indicating the type of regression plot (bit 1: linear, bit 2: loess).
 #'
-#' @return 
+#' @return
 #'   No return value, called from shiny applications for side effects.
 #'
 #' @examples
@@ -485,8 +637,8 @@ setDistribType <- function(id, distribType) {
 #'        scatterPlotMatrix(iris)
 #'      })
 #'      observe({
-#'        linearFlag <- ifelse(input$linearRegressionCB, 1, 0)    
-#'        loessFlag <- ifelse(input$loessCB, 2, 0)    
+#'        linearFlag <- ifelse(input$linearRegressionCB, 1, 0)
+#'        loessFlag <- ifelse(input$loessCB, 2, 0)
 #'        scatterPlotMatrix::setRegressionType("spMatrix", linearFlag + loessFlag)
 #'      })
 #'    }
@@ -496,17 +648,17 @@ setDistribType <- function(id, distribType) {
 #'
 #' @export
 setRegressionType <- function(id, regressionType) {
-  method <- "setRegressionType"
+  method <- "setRegressionType" # nolint
   callJS()
 }
 
 #' Tells which type of correlation plot to use.
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param corrPlotType 
-#'   One of the available correlation plot types (Circles, Text).
+#' @param corrPlotType
+#'   One of the available correlation plot types (Empty, Circles, Text, AbsText).
 #'
-#' @return 
+#' @return
 #'   No return value, called from shiny applications for side effects.
 #'
 #' @examples
@@ -516,9 +668,14 @@ setRegressionType <- function(id, regressionType) {
 #'
 #'    ui <- fluidPage(
 #'      selectInput(
-#'        "corrPlotTypeSelect", 
-#'        "Correlation Plot Type:", 
-#'        choices = list("Circles" = "Circles", "Text" = "Text"), 
+#'        "corrPlotTypeSelect",
+#'        "Correlation Plot Type:",
+#'        choices = list(
+#'          "Empty" = "Empty",
+#'          "Circles" = "Circles",
+#'          "Text" = "Text",
+#'          "AbsText" = "AbsText"
+#'        ),
 #'        selected = "Circles"
 #'      ),
 #'      p("The selector controls the type of correlation to use"),
@@ -539,17 +696,17 @@ setRegressionType <- function(id, regressionType) {
 #'
 #' @export
 setCorrPlotType <- function(id, corrPlotType) {
-  method <- "setCorrPlotType"
+  method <- "setCorrPlotType" # nolint
   callJS()
 }
 
 #' Tells which color scale to use for correlation plots.
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param corrPlotCsId 
-#'   One of the available color scale ids 
-#'   ("Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault", 
-#'    "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd", 
+#' @param corrPlotCsId
+#'   One of the available color scale ids
+#'   ("Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault",
+#'    "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd",
 #'    "PuBuGn","PuBu", "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd").
 #'
 #' @return No return value, called from shiny applications for side effects.
@@ -561,22 +718,22 @@ setCorrPlotType <- function(id, corrPlotType) {
 #'
 #'    ui <- fluidPage(
 #'      selectInput(
-#'        "corrPlotCsSelect", 
-#'        "Correlation Plot Color Scale:", 
+#'        "corrPlotCsSelect",
+#'        "Correlation Plot Color Scale:",
 #'        choices = list(
-#'          "Viridis" = "Viridis", "Inferno" = "Inferno", "Magma" = "Magma", 
-#'          "Plasma" = "Plasma", "Warm" = "Warm", "Cool" = "Cool", "Rainbow" ="Rainbow", 
-#'          "CubehelixDefault" = "CubehelixDefault", "Blues" = "Blues", 
-#'          "Greens" = "Greens", "Greys" = "Greys", "Oranges" = "Oranges", 
-#'          "Purples" = "Purples", "Reds" = "Reds", "BuGn" = "BuGn", "BuPu" = "BuPu", 
-#'          "GnBu" = "GnBu", "OrRd" = "OrRd", "PuBuGn" = "PuBuGn", "PuBu" = "PuBu", 
-#'          "PuRd" = "PuRd", "RdBu" = "RdBu", "RdPu" = "RdPu", "YlGnBu" = "YlGnBu", 
+#'          "Viridis" = "Viridis", "Inferno" = "Inferno", "Magma" = "Magma",
+#'          "Plasma" = "Plasma", "Warm" = "Warm", "Cool" = "Cool", "Rainbow" ="Rainbow",
+#'          "CubehelixDefault" = "CubehelixDefault", "Blues" = "Blues",
+#'          "Greens" = "Greens", "Greys" = "Greys", "Oranges" = "Oranges",
+#'          "Purples" = "Purples", "Reds" = "Reds", "BuGn" = "BuGn", "BuPu" = "BuPu",
+#'          "GnBu" = "GnBu", "OrRd" = "OrRd", "PuBuGn" = "PuBuGn", "PuBu" = "PuBu",
+#'          "PuRd" = "PuRd", "RdBu" = "RdBu", "RdPu" = "RdPu", "YlGnBu" = "YlGnBu",
 #'          "YlGn" = "YlGn", "YlOrBr" = "YlOrBr", "YlOrRd" = "YlOrRd"
-#'        ), 
+#'        ),
 #'        selected = "Plasma"
 #'      ),
-#'      p("The selector controls the color scale to use for correlation plot 
-#'         when plot type is 'Text'"),
+#'      p("The selector controls the color scale to use for correlation plot
+#'         when plot type is 'Text' or 'AbsText'"),
 #'      scatterPlotMatrixOutput("spMatrix")
 #'    )
 #'
@@ -594,17 +751,17 @@ setCorrPlotType <- function(id, corrPlotType) {
 #'
 #' @export
 setCorrPlotCS <- function(id, corrPlotCsId) {
-  method <- "setCorrPlotCS"
+  method <- "setCorrPlotCS" # nolint
   callJS()
 }
 
 #' Tells which color scale to use for continuous columns.
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param continuousCsId 
-#'   One of the available color scale ids 
-#'   ("Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault", 
-#'    "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd", 
+#' @param continuousCsId
+#'   One of the available color scale ids
+#'   ("Viridis", "Inferno", "Magma", "Plasma", "Warm", "Cool", "Rainbow", "CubehelixDefault",
+#'    "Blues","Greens", "Greys", "Oranges", "Purples", "Reds", "BuGn", "BuPu", "GnBu", "OrRd",
 #'    "PuBuGn","PuBu", "PuRd", "RdBu", "RdPu", "YlGnBu", "YlGn", "YlOrBr", "YlOrRd").
 #'
 #' @return No return value, called from shiny applications for side effects.
@@ -616,18 +773,18 @@ setCorrPlotCS <- function(id, corrPlotCsId) {
 #'
 #'    ui <- fluidPage(
 #'      selectInput(
-#'        "continuousCsSelect", 
-#'        "Continuous Color Scale:", 
+#'        "continuousCsSelect",
+#'        "Continuous Color Scale:",
 #'        choices = list(
-#'          "Viridis" = "Viridis", "Inferno" = "Inferno", "Magma" = "Magma", 
-#'          "Plasma" = "Plasma", "Warm" = "Warm", "Cool" = "Cool", "Rainbow" ="Rainbow", 
-#'          "CubehelixDefault" = "CubehelixDefault", "Blues" = "Blues", 
-#'          "Greens" = "Greens", "Greys" = "Greys", "Oranges" = "Oranges", 
-#'          "Purples" = "Purples", "Reds" = "Reds", "BuGn" = "BuGn", "BuPu" = "BuPu", 
-#'          "GnBu" = "GnBu", "OrRd" = "OrRd", "PuBuGn" = "PuBuGn", "PuBu" = "PuBu", 
-#'          "PuRd" = "PuRd", "RdBu" = "RdBu", "RdPu" = "RdPu", "YlGnBu" = "YlGnBu", 
+#'          "Viridis" = "Viridis", "Inferno" = "Inferno", "Magma" = "Magma",
+#'          "Plasma" = "Plasma", "Warm" = "Warm", "Cool" = "Cool", "Rainbow" ="Rainbow",
+#'          "CubehelixDefault" = "CubehelixDefault", "Blues" = "Blues",
+#'          "Greens" = "Greens", "Greys" = "Greys", "Oranges" = "Oranges",
+#'          "Purples" = "Purples", "Reds" = "Reds", "BuGn" = "BuGn", "BuPu" = "BuPu",
+#'          "GnBu" = "GnBu", "OrRd" = "OrRd", "PuBuGn" = "PuBuGn", "PuBu" = "PuBu",
+#'          "PuRd" = "PuRd", "RdBu" = "RdBu", "RdPu" = "RdPu", "YlGnBu" = "YlGnBu",
 #'          "YlGn" = "YlGn", "YlOrBr" = "YlOrBr", "YlOrRd" = "YlOrRd"
-#'        ), 
+#'        ),
 #'        selected = "Viridis"
 #'      ),
 #'      p("The selector controls the colors used when reference column is of type continuous"),
@@ -648,17 +805,17 @@ setCorrPlotCS <- function(id, corrPlotCsId) {
 #'
 #' @export
 setContinuousColorScale <- function(id, continuousCsId) {
-  method <- "setContinuousColorScale"
+  method <- "setContinuousColorScale" # nolint
   callJS()
 }
 
 #' Tells which color scale to use for categorical columns.
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param categoricalCsId 
+#' @param categoricalCsId
 #'   One of the available color scale ids (Category10, Accent, Dark2, Paired, Set1).
 #'
-#' @return 
+#' @return
 #'   No return value, called from shiny applications for side effects.
 #'
 #' @examples
@@ -668,12 +825,12 @@ setContinuousColorScale <- function(id, continuousCsId) {
 #'
 #'    ui <- fluidPage(
 #'      selectInput(
-#'        "categoricalCsSelect", 
-#'        "Categorical Color Scale:", 
+#'        "categoricalCsSelect",
+#'        "Categorical Color Scale:",
 #'        choices = list(
-#'          "Category10" = "Category10", "Accent" = "Accent", "Dark2" = "Dark2", 
+#'          "Category10" = "Category10", "Accent" = "Accent", "Dark2" = "Dark2",
 #'          "Paired" = "Paired", "Set1" = "Set1"
-#'        ), 
+#'        ),
 #'        selected = "Category10"
 #'      ),
 #'      p("The selector controls the colors used when reference column is of type categorical"),
@@ -694,21 +851,79 @@ setContinuousColorScale <- function(id, continuousCsId) {
 #'
 #' @export
 setCategoricalColorScale <- function(id, categoricalCsId) {
-  method <- "setCategoricalColorScale"
+  method <- "setCategoricalColorScale" # nolint
   callJS()
 }
 
-#' Column visibitity
+#' Cutoffs values
+#'
+#' Tells which cutoffs to use for each pair of columns.
+#'
+#' It's possible to filter some points by defining cutoffs to apply to columns.
+#'
+#' @param id
+#'   output variable to read from (id which references the requested plot)
+#' @param cutoffs
+#'   List of 'SpCutoff'; a 'SpCutoff' is a list defining a 'xDim', 'yDim' and a list of 'xyCutoff';
+#'   a 'xyCutoff' is a pair of 'cutoff' (one for x axis, one for y axis);
+#'   a 'cutoff' is a list containing two values (min and max values)
+#'   or \code{NULL} if there is no cutoff to apply for this axis;
+#'   \code{NULL} is allowed, meaning there is no cutoff to apply.
+#'
+#' @return
+#'   No return value, called from shiny applications for side effects.
+#'
+#' @examples
+#'  if(interactive()) {
+#'    library(shiny)
+#'    library(scatterPlotMatrix)
+#'
+#'    ui <- fluidPage(
+#'      checkboxInput("setosaCB", "Setosa", TRUE),
+#'      checkboxInput("versicolorCB", "Versicolor", TRUE),
+#'      checkboxInput("viginicaCB", "Viginica", TRUE),
+#'      scatterPlotMatrixOutput("spMatrix")
+#'    )
+#'
+#'    server <- function(input, output, session) {
+#'      output$spMatrix <- renderScatterPlotMatrix({
+#'        scatterPlotMatrix(
+#'          data = iris,
+#'          zAxisDim = "Species"
+#'        )
+#'      })
+#'
+#'      observe({
+#'        speciesCBs = c(input$setosaCB, input$versicolorCB, input$viginicaCB)
+#'        toKeepIndexes <- Filter(function(i) speciesCBs[i], 1:length(speciesCBs))
+#'        xyCutoffs <- sapply(toKeepIndexes, function(i) {
+#'          list(list(NULL, c(i - 1.1, i - 0.9)))
+#'        })
+#'        scatterPlotMatrix::setCutoffs("spMatrix", list(
+#'          list(xDim="Sepal.Length", yDim="Species", xyCutoffs = xyCutoffs)
+#'        ))
+#'      })
+#'    }
+#'    shinyApp(ui, server)
+#'  }
+#'
+#' @export
+setCutoffs <- function(id, cutoffs) {
+  method <- "setCutoffs" # nolint
+  callJS()
+}
+
+#' Column visibility
 #'
 #' Tells which columns have to be visible.
 #'
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param keptColumns 
-#'   Vector of boolean (one for each data column), \code{FALSE} if column has to be hidden. 
+#' @param keptColumns
+#'   Vector of boolean (one for each data column), \code{FALSE} if column has to be hidden.
 #'   A named list can also be provided to only indicate which columns must be assigned to a new visibility.
 #'
-#' @return 
+#' @return
 #'   No return value, called from shiny applications for side effects.
 #'
 #' @examples
@@ -728,7 +943,7 @@ setCategoricalColorScale <- function(id, categoricalCsId) {
 #'      })
 #'      observeEvent(input$hideColumnsCB, {
 #'        keptColumns <- vapply(
-#'          1:ncol(iris), 
+#'          1:ncol(iris),
 #'          function(i) {
 #'            return(ifelse(input$hideColumnsCB, ncol(iris) - i >= 2, TRUE))
 #'          },
@@ -743,17 +958,18 @@ setCategoricalColorScale <- function(id, categoricalCsId) {
 #'
 #' @export
 setKeptColumns <- function(id, keptColumns) {
-  method <- "setKeptColumns"
+  method <- "setKeptColumns" # nolint
   callJS()
 }
 
-#' This function allows to set the type of interaction; three types of mouse interactions are available ('tooltip', 'filter' or 'zoom').
-#' @param id 
+#' This function allows to set the type of interaction;
+#' three types of mouse interactions are available ('tooltip', 'filter' or 'zoom').
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param interactionType 
+#' @param interactionType
 #'   Type of mouse interaction.
 #'
-#' @return 
+#' @return
 #'   No return value, called from shiny applications for side effects.
 #'
 #' @examples
@@ -763,8 +979,8 @@ setKeptColumns <- function(id, keptColumns) {
 #'
 #'    ui <- fluidPage(
 #'      selectInput(
-#'        "mouseMode", 
-#'        "Mouse Interactions:", 
+#'        "mouseMode",
+#'        "Mouse Interactions:",
 #'        c("Tooltip" = "tooltip", "Filter" = "filter", "Zoom" = "zoom")
 #'      ),
 #'      p("The selector controls the type of mouse interactions with the scatterPlotMatrix"),
@@ -785,17 +1001,17 @@ setKeptColumns <- function(id, keptColumns) {
 #'
 #' @export
 changeMouseMode <- function(id, interactionType) {
-  method <- "changeMouseMode"
+  method <- "changeMouseMode" # nolint
   callJS()
 }
 
 #' Tells which dim is to display on Z axis.
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param dim 
+#' @param dim
 #'   Is to display on X axis.
 #'
-#' @return 
+#' @return
 #'   No return value, called from shiny applications for side effects.
 #'
 #' @examples
@@ -821,7 +1037,7 @@ changeMouseMode <- function(id, interactionType) {
 #'      output$spMatrix <- renderScatterPlotMatrix({
 #'        scatterPlotMatrix(iris)
 #'      })
-#'        
+#'
 #'      observe({
 #'        scatterPlotMatrix::setZAxis("spMatrix", if (input$zAxisUsedCB) input$zAxisSelect else NULL)
 #'      })
@@ -832,23 +1048,23 @@ changeMouseMode <- function(id, interactionType) {
 #'
 #' @export
 setZAxis <- function(id, dim) {
-  method <- "setZAxis"
+  method <- "setZAxis" # nolint
   callJS()
 }
 
-#' Asks to retrieve the plot configuration. 
+#' Asks to retrieve the plot configuration.
 #' Result will be sent through a reactive input.
-#' @param id 
+#' @param id
 #'   Output variable to read from (id which references the requested plot).
-#' @param configInputId 
-#'   Reactive input to write to. 
+#' @param configInputId
+#'   Reactive input to write to.
 #'
-#' @return 
+#' @return
 #'   No return value, called from shiny applications for side effects.
 #'
 #' @export
 getPlotConfig <- function(id, configInputId) {
-  method <- "getPlotConfig"
+  method <- "getPlotConfig" # nolint
   callJS()
 }
 
